@@ -31,6 +31,8 @@
 
 #include <bonobo.h>
 
+#include <libgnomevfs/gnome-vfs.h>
+
 struct _GgvPostScriptViewPrivate {
 	GtkWidget *gs;
 
@@ -46,8 +48,6 @@ struct _GgvPostScriptViewPrivate {
 	GgvMsgWindow *msg_win;
 	GgvSidebar *sidebar;
 
-	GtkWidget *save_file_sel;
-
 	BonoboPropertyBag     *property_bag;
 	BonoboPropertyControl *property_control;
 
@@ -314,6 +314,38 @@ load_ps(GgvPostScriptView *ps_view, const gchar *fname)
 	notify_status_change(ps_view);
 }
 
+static void
+ps_view_clean_tmp_file(GgvPostScriptView *ps_view)
+{
+	/* copy stream to a tmp file */
+	if(ps_view->priv->tmp_name != NULL) {
+		unlink(ps_view->priv->tmp_name);
+		g_free(ps_view->priv->tmp_name);
+		ps_view->priv->tmp_name = NULL;
+	}
+}
+
+static FILE *
+ps_view_get_tmp_file(GgvPostScriptView *ps_view)
+{
+	int fd;
+	FILE *tmpfile;
+
+	ps_view_clean_tmp_file(ps_view);
+	ps_view->priv->tmp_name = g_strconcat(g_get_tmp_dir(), "/ggvXXXXXX", NULL);
+	if((fd = mkstemp(ps_view->priv->tmp_name)) < 0) {
+		g_free(ps_view->priv->tmp_name),
+		ps_view->priv->tmp_name = NULL;
+		return;
+	}
+	tmpfile = fdopen(fd, "w");
+	if(!tmpfile) {
+		close(fd);
+		return NULL;
+	}
+	return tmpfile;
+}
+
 /*
  * Loads an postscript document from a Bonobo_Stream
  */
@@ -335,23 +367,8 @@ load_ps_from_stream (BonoboPersistStream *ps,
 
 	ps_view = GGV_POSTSCRIPT_VIEW(data);
 
-	/* copy stream to a tmp file */
-	if(ps_view->priv->tmp_name != NULL) {
-		unlink(ps_view->priv->tmp_name);
-		g_free(ps_view->priv->tmp_name);
-		ps_view->priv->tmp_name = NULL;
-	}
-	ps_view->priv->tmp_name = g_strconcat(g_get_tmp_dir(), "/ggvXXXXXX", NULL);
-	if((fd = mkstemp(ps_view->priv->tmp_name)) < 0) {
-		g_free(ps_view->priv->tmp_name),
-		ps_view->priv->tmp_name = NULL;
-		return;
-	}
-	tmpfile = fdopen(fd, "w");
-	if(!tmpfile) {
-		close(fd);
-		return;
-	}
+	ps_view_clean_tmp_file(ps_view);
+	tmpfile = ps_view_get_tmp_file(ps_view);
 
 	do {
 		Bonobo_Stream_read (stream, 32768, &buffer, ev);
@@ -483,14 +500,51 @@ static gint
 load_ps_from_file (BonoboPersistFile *pf, const CORBA_char *filename,
 				   CORBA_Environment *ev, void *data)
 {
+	GnomeVFSResult result;
+	GnomeVFSHandle *handle;
+	GnomeVFSURI     *uri;
+	GnomeVFSFileSize bytes_read = 0;
+	guchar *buffer;
 	GgvPostScriptView *ps_view;
+	FILE *tmpfile;
 
 	g_return_val_if_fail (data != NULL, -1);
 	g_return_val_if_fail (GGV_IS_POSTSCRIPT_VIEW (data), -1);
 
-	ps_view = GGV_POSTSCRIPT_VIEW (data);
+	ps_view = GGV_POSTSCRIPT_VIEW(data);
 
-	load_ps(ps_view, filename);
+	uri = gnome_vfs_uri_new(filename);
+	/* open uri */
+	result = gnome_vfs_open_uri(&handle, uri, GNOME_VFS_OPEN_READ);
+	gnome_vfs_uri_unref(uri);
+	if(result != GNOME_VFS_OK) {
+		return -1;
+	}
+
+	ps_view_clean_tmp_file(ps_view);
+	tmpfile = ps_view_get_tmp_file(ps_view);
+
+	buffer = g_new0(guchar, 4096);
+	do {
+		result = gnome_vfs_read (handle, buffer,
+					 4096, &bytes_read);
+		if (result != GNOME_VFS_OK)
+			break;
+
+		if (bytes_read > 0)
+			if(fwrite(buffer, 1, bytes_read, tmpfile) != bytes_read) {
+				result = GNOME_VFS_ERROR_GENERIC;
+				break;
+			}
+	} while(TRUE);
+
+	gnome_vfs_close(handle);
+	fclose(tmpfile);
+
+	if(result != GNOME_VFS_ERROR_EOF)
+		return -1;
+	
+	load_ps(ps_view, ps_view->priv->tmp_name);
 
 	return 0;
 }
@@ -680,74 +734,30 @@ ggv_postscript_view_print(GgvPostScriptView *ps_view, gchar *ps)
 	g_strfreev(argv);
 }
 
-static void
-save_file_sel_unmap(GtkWidget *widget)
-{
-	ggv_get_window_size(widget, &file_sel_width, &file_sel_height);
-}
-
 static gboolean
-save_file_sel_delete_event(GtkWidget *widget, GdkEventAny *e, gpointer data)
+ps_view_save_doc(GgvPostScriptView *ps_view, const gchar *fname,
+				 const gchar *doc, gint len)
 {
-	gtk_widget_hide(widget);
-	return TRUE;
-}
+	GnomeVFSResult result;
+	GnomeVFSHandle *handle;
+	GnomeVFSURI *uri;
+	GnomeVFSFileSize written;
 
-static void
-save_file_sel_ok_clicked(GtkWidget *widget, gpointer data)
-{
-	GgvPostScriptView *ps_view = GGV_POSTSCRIPT_VIEW(data);
-	gint *active_rows;
-	const gchar *fname;
-	gchar *doc;
-	FILE *f;
-
-	active_rows = ggv_sidebar_get_active_list(ps_view->priv->sidebar);
-	doc = ggv_postscript_view_get_ps(ps_view, active_rows);
-	g_free(active_rows);
-
-	/* get file name */
-	gtk_widget_hide(ps_view->priv->save_file_sel);
-
-	if(doc != NULL) {
-		fname = gtk_file_selection_get_filename(GTK_FILE_SELECTION(ps_view->priv->save_file_sel));
-
-		if(ggv_file_readable(fname)) {
-			GtkWidget *dlg;
-		
-			dlg = gtk_message_dialog_new(GTK_WINDOW(gtk_widget_get_toplevel(ps_view->priv->gs)),
-										 GTK_DIALOG_MODAL,
-										 GTK_MESSAGE_QUESTION,
-										 GTK_BUTTONS_YES_NO,
-										 _("A file with this name already exists.\n"
-										   "Do you want to overwrite it?"));
-			gtk_widget_show(dlg);
-			switch(gtk_dialog_run(GTK_DIALOG(dlg))) {
-			case GTK_RESPONSE_NO:
-			case GTK_RESPONSE_DELETE_EVENT:
-				gtk_widget_destroy(dlg);
-				g_free(doc);
-				return;
-			default:
-				break;
-			}
-			gtk_widget_destroy(dlg);
-		}
-		
-		if(fname && (f = fopen(fname, "w")) != NULL) {
-			fwrite(doc, 1, strlen(doc), f);
-			fclose(f);
-		}
-		g_free(doc);
+	uri = gnome_vfs_uri_new (fname);
+	result = gnome_vfs_create_uri (&handle, uri,
+								   GNOME_VFS_OPEN_WRITE, FALSE, 0644);
+	gnome_vfs_uri_unref(uri);
+	if (result != GNOME_VFS_OK) {
+		return FALSE;
 	}
-}
 
-static void
-save_file_sel_cancel_clicked(GtkWidget *widget, gpointer data)
-{
-	GgvPostScriptView *ps_view = GGV_POSTSCRIPT_VIEW(data);
+	result = gnome_vfs_write(handle, doc, len, &written);
+	gnome_vfs_close(handle);
 
-	gtk_widget_hide(ps_view->priv->save_file_sel);
+	if(result != GNOME_VFS_OK || written != len)
+		return FALSE;
+	else
+		return TRUE;
 }
 
 static void
@@ -755,7 +765,8 @@ verb_FileSaveMarked(BonoboUIComponent *uic, gpointer data, const char *cname)
 {
 	GgvPostScriptView *ps_view = GGV_POSTSCRIPT_VIEW(data);
 	gint *active_rows;
-	
+	gchar *doc, *fname;
+
 	active_rows = ggv_sidebar_get_active_list(ps_view->priv->sidebar);
 	if(active_rows[0] == -1) {
 		GtkWidget *dlg;
@@ -779,37 +790,39 @@ verb_FileSaveMarked(BonoboUIComponent *uic, gpointer data, const char *cname)
 	}
 	g_free(active_rows);
 	
-	if(!ps_view->priv->save_file_sel) {
-		gchar *home_path = g_strconcat(g_get_home_dir(), "/", NULL);
-		ps_view->priv->save_file_sel = gtk_file_selection_new(_("Select a file to save pages as"));
-		gtk_file_selection_set_filename(GTK_FILE_SELECTION(ps_view->priv->save_file_sel),
-										home_path);
-		g_free(home_path);
-		gtk_window_set_transient_for(GTK_WINDOW(ps_view->priv->save_file_sel),
-									 GTK_WINDOW(gtk_widget_get_toplevel(ps_view->priv->gs)));
-		gtk_file_selection_hide_fileop_buttons(GTK_FILE_SELECTION(ps_view->priv->save_file_sel));
-		gtk_window_set_resizable(GTK_WINDOW(ps_view->priv->save_file_sel), TRUE);
-		g_signal_connect(G_OBJECT(GTK_FILE_SELECTION(ps_view->priv->save_file_sel)->ok_button),
-						 "clicked", G_CALLBACK(save_file_sel_ok_clicked), ps_view);
-		g_signal_connect(G_OBJECT(GTK_FILE_SELECTION(ps_view->priv->save_file_sel)->cancel_button),
-						 "clicked", G_CALLBACK(save_file_sel_cancel_clicked), ps_view);
-		g_signal_connect(G_OBJECT(ps_view->priv->save_file_sel), "delete_event",
-						 G_CALLBACK(save_file_sel_delete_event), ps_view);
-		g_signal_connect(G_OBJECT(ps_view->priv->save_file_sel), "unmap",
-						 G_CALLBACK(save_file_sel_unmap), NULL);
-	}
-	if(!GTK_WIDGET_VISIBLE(ps_view->priv->save_file_sel)) {
-		gtk_widget_set_usize(GTK_WIDGET(ps_view->priv->save_file_sel),
-							 file_sel_width, file_sel_height);
-		if(GTK_GS(ps_view->priv->gs)->gs_filename) {
-			gchar *path = g_path_get_dirname(GTK_GS(ps_view->priv->gs)->gs_filename);
-			gchar *path2 = g_strconcat(path, "/", NULL);
-			g_free(path);
-			gtk_file_selection_set_filename(GTK_FILE_SELECTION(ps_view->priv->save_file_sel), path2);
-			g_free(path2);
+	active_rows = ggv_sidebar_get_active_list(ps_view->priv->sidebar);
+	doc = ggv_postscript_view_get_ps(ps_view, active_rows);
+	g_free(active_rows);
+
+	if(doc != NULL) {
+		fname = bonobo_file_selector_open(GTK_WINDOW(gtk_widget_get_toplevel(ps_view->priv->gs)),
+										  TRUE, _("Select a file to save pages as"), NULL, NULL);
+		if(fname) {
+			if(ggv_file_readable(fname)) {
+				GtkWidget *dlg;
+		
+				dlg = gtk_message_dialog_new(GTK_WINDOW(gtk_widget_get_toplevel(ps_view->priv->gs)),
+											 GTK_DIALOG_MODAL,
+											 GTK_MESSAGE_QUESTION,
+											 GTK_BUTTONS_YES_NO,
+											 _("A file with this name already exists.\n"
+											   "Do you want to overwrite it?"));
+				gtk_widget_show(dlg);
+				switch(gtk_dialog_run(GTK_DIALOG(dlg))) {
+				case GTK_RESPONSE_NO:
+				case GTK_RESPONSE_DELETE_EVENT:
+					gtk_widget_destroy(dlg);
+					g_free(doc);
+					return;
+				default:
+					break;
+				}
+				gtk_widget_destroy(dlg);
+			}
+			ps_view_save_doc(ps_view, fname, doc, strlen(doc));
+			g_free(fname);
 		}
-		gtk_widget_show(ps_view->priv->save_file_sel);
-		ggv_raise_and_focus_widget(ps_view->priv->save_file_sel);
+		g_free(doc);
 	}
 }
 
@@ -1621,6 +1634,7 @@ ggv_postscript_view_destroy(BonoboObject *object)
 		bonobo_object_unref(BONOBO_OBJECT(ps_view->priv->sidebar));
 		ps_view->priv->sidebar = NULL;
 	}
+	ps_view_clean_tmp_file(ps_view);
 
 	if (BONOBO_OBJECT_CLASS (ggv_postscript_view_parent_class)->destroy)
 		BONOBO_OBJECT_CLASS(ggv_postscript_view_parent_class)->destroy(object);
@@ -1690,6 +1704,9 @@ impl_GNOME_GGV_PostScriptView_reload(PortableServer_Servant servant,
 
 	ps_view = GGV_POSTSCRIPT_VIEW(bonobo_object_from_servant(servant));
 	gtk_gs_reload(GTK_GS(ps_view->priv->gs));
+	notify_page_count_change(ps_view);
+	notify_page_change(ps_view);
+	notify_status_change(ps_view);
 }
 
 static void
