@@ -70,15 +70,16 @@ ggv_window_drag_data_received(GtkWidget *widget,
                         newwin = GTK_WIDGET(win);
                 else
                         newwin = NULL;
-                while(*uri) {
-                        if(!g_ascii_strncasecmp("file:", *uri, 5)) {
-                                if(newwin == NULL) {
-                                        newwin = ggv_window_new();
-                                        gtk_widget_show(newwin);
-                                }
-                                ggv_window_load(GGV_WINDOW(newwin), (*uri) + 5);
-                                newwin = NULL;
+                while(*uri && **uri != '\0') {
+#ifdef _DEBUG
+                        g_message("URI %s\n", *uri);
+#endif /* _DEBUG */
+                        if(newwin == NULL) {
+                                newwin = ggv_window_new();
+                                gtk_widget_show(newwin);
                         }
+                        ggv_window_load(GGV_WINDOW(newwin), (*uri));
+                        newwin = NULL;
                         uri++;
                 }
                 g_strfreev(win->uris_to_open);
@@ -140,6 +141,8 @@ sync_toolbar_style(GgvWindow *win)
                                 tbstyle = "both";
                         else if(!strcmp(gnome_tbstyle, "text"))
                                 tbstyle = "text";
+                        else if(!strcmp(gnome_tbstyle, "both_horiz"))
+                                tbstyle = "both_horiz";
                         else
                                 tbstyle = "both";
                         g_free(gnome_tbstyle);
@@ -175,9 +178,6 @@ control_property_changed_handler(BonoboListener    *listener,
         GgvWindow *win = GGV_WINDOW(data);
 
         if(!g_ascii_strcasecmp(event_name, "Bonobo/Property:change:title")) {
-                gchar *title = g_strconcat(_("GGV: "), BONOBO_ARG_GET_STRING(any), NULL);
-                gtk_window_set_title(GTK_WINDOW(win), title);
-                g_free(title);
         }
         else if(!g_ascii_strcasecmp(event_name, "Bonobo/Property:change:status")) {
                 gnome_appbar_set_status(GNOME_APPBAR(win->statusbar),
@@ -194,6 +194,9 @@ control_property_changed_handler(BonoboListener    *listener,
                         BonoboArg *arg;
                         gchar *msg;
 
+                        gtk_window_set_title(GTK_WINDOW(win),
+                                             _("GGV: no document loaded"));
+
                         arg = bonobo_pbclient_get_value(win->pb, "status",
                                                         NULL, NULL);
                         if(arg == NULL)
@@ -220,80 +223,29 @@ control_property_changed_handler(BonoboListener    *listener,
         }
 }
 
-static void
-file_sel_unmap(GtkWidget *widget)
-{
-        ggv_get_window_size(widget, &file_sel_width, &file_sel_height);
-}
-
-static gboolean
-file_sel_delete_event(GtkWidget *widget, GdkEventAny *e, gpointer data)
-{
-        gtk_widget_hide(widget);
-        return TRUE;
-}
-
-static void
-file_sel_ok_clicked(GtkWidget *widget, gpointer data)
-{
-        GgvWindow *win = GGV_WINDOW(data);
-        const gchar *fname;
-
-        gtk_widget_hide(win->file_sel);
-        fname = gtk_file_selection_get_filename(GTK_FILE_SELECTION(win->file_sel));
-        if(!ggv_window_load(win, fname)) {
-                GtkWidget *dlg;
-                dlg = gtk_message_dialog_new(GTK_WINDOW(win),
-                                             GTK_DIALOG_MODAL,
-                                             GTK_MESSAGE_ERROR,
-                                             GTK_BUTTONS_OK,
-                                             _("Unable to load file:\n%s"),
-                                             fname);
-                gtk_widget_show(dlg);
-                gtk_dialog_run(GTK_DIALOG(dlg));
-                gtk_widget_destroy(dlg);
-        }
-}
-
-static void
-file_sel_cancel_clicked(GtkWidget *widget, gpointer data)
-{
-        GgvWindow *win = GGV_WINDOW(data);
-
-        gtk_widget_hide(win->file_sel);
-}
-
 static void
 verb_FileOpen(BonoboUIComponent *uic, gpointer data, const char *cname)
 {
         GgvWindow *win = GGV_WINDOW(data);
+        gchar *fname, *escaped_fname;
 
-        if(!win->file_sel) {
-                win->file_sel = gtk_file_selection_new(_("Select a PostScript document"));
-                gtk_window_set_transient_for(GTK_WINDOW(win->file_sel), GTK_WINDOW(win));
-                gtk_file_selection_hide_fileop_buttons(GTK_FILE_SELECTION(win->file_sel));
-                gtk_window_set_resizable(GTK_WINDOW(win->file_sel), TRUE);
-                g_signal_connect(G_OBJECT(GTK_FILE_SELECTION(win->file_sel)->ok_button),
-                                 "clicked", G_CALLBACK(file_sel_ok_clicked), win);
-                g_signal_connect(G_OBJECT(GTK_FILE_SELECTION(win->file_sel)->cancel_button),
-                                 "clicked", G_CALLBACK(file_sel_cancel_clicked), win);
-                g_signal_connect(G_OBJECT(win->file_sel), "delete_event",
-                                 G_CALLBACK(file_sel_delete_event), win);
-                g_signal_connect(G_OBJECT(win->file_sel), "unmap",
-                                 G_CALLBACK(file_sel_unmap), NULL);
-        }
-        if(!GTK_WIDGET_VISIBLE(win->file_sel)) {
-                gtk_widget_set_usize(GTK_WIDGET(win->file_sel),
-                                     file_sel_width, file_sel_height);
-                if(win->filename) {
-                        gchar *path = g_path_get_dirname(win->filename);
-			gchar *path2 = g_strconcat(path, "/", NULL);
-			g_free(path);
-                        gtk_file_selection_set_filename(GTK_FILE_SELECTION(win->file_sel), path2);
-                        g_free(path2);
+        fname = bonobo_file_selector_open(GTK_WINDOW(win), TRUE,
+                                          _("Select a PostScript document"),
+                                          NULL, NULL);
+        if(fname) {
+                if(!ggv_window_load(win, fname)) {
+                        GtkWidget *dlg;
+                        dlg = gtk_message_dialog_new(GTK_WINDOW(win),
+                                                     GTK_DIALOG_MODAL,
+                                                     GTK_MESSAGE_ERROR,
+                                                     GTK_BUTTONS_OK,
+                                                     _("Unable to load file:\n%s"),
+                                                     fname);
+                        gtk_widget_show(dlg);
+                        gtk_dialog_run(GTK_DIALOG(dlg));
+                        gtk_widget_destroy(dlg);
                 }
-                gtk_widget_show(win->file_sel);
-		ggv_raise_and_focus_widget(win->file_sel);
+                g_free(fname);
         }
 }
 
@@ -314,8 +266,6 @@ verb_FileReload(BonoboUIComponent *uic, gpointer data, const char *cname)
                 if(BONOBO_EX(&ev))
                         g_warning("Could not invoke reload method.");
                 bonobo_object_release_unref(ps_view, &ev);
-                if(BONOBO_EX(&ev))
-                        g_warning("Could not invoke reload method.");
         }
         CORBA_exception_free(&ev);
 }
@@ -468,15 +418,16 @@ verb_DnDNewWindow(BonoboUIComponent *uic, gpointer user_data, const char *cname)
 {
         GgvWindow *win = GGV_WINDOW(user_data);
         gchar **uri;
+        GtkWidget *newwin;
 
         uri = win->uris_to_open;
-        while(*uri) {
-                GtkWidget *newwin;
-                if(!g_ascii_strncasecmp("file:", *uri, 5)) {
-                        newwin = ggv_window_new();
-                        gtk_widget_show(newwin);
-                        ggv_window_load(GGV_WINDOW(newwin), (*uri) + 5);
-                }
+        while(*uri && **uri != '\0') {
+#ifdef _DEBUG
+                g_message("URI %s", *uri);
+#endif /* _DEBUG */
+                newwin = ggv_window_new();
+                gtk_widget_show(newwin);
+                ggv_window_load(GGV_WINDOW(newwin), (*uri));
                 uri++;
         }
         g_strfreev(win->uris_to_open);
@@ -492,15 +443,16 @@ verb_DnDThisWindow(BonoboUIComponent *uic, gpointer user_data, const char *cname
 
         uri = win->uris_to_open;
         newwin = GTK_WIDGET(win);
-        while(*uri) {
-                if(!g_ascii_strncasecmp("file:", *uri, 5)) {
-                        if(newwin == NULL) {
-                                newwin = ggv_window_new();
-                                gtk_widget_show(newwin);
-                        }
-                        ggv_window_load(GGV_WINDOW(newwin), (*uri) + 5);
-                        newwin = NULL;
+        while(*uri && **uri != '\0') {
+#ifdef _DEBUG
+                g_message("URI %s", *uri);
+#endif /* _DEBUG */
+                if(newwin == NULL) {
+                        newwin = ggv_window_new();
+                        gtk_widget_show(newwin);
                 }
+                ggv_window_load(GGV_WINDOW(newwin), (*uri));
+                newwin = NULL;
                 uri++;
         }
         g_strfreev(win->uris_to_open);
@@ -538,11 +490,9 @@ control_frame_activate_uri(BonoboControlFrame *control_frame, const char *uri,
 
 	g_return_if_fail(uri != NULL);
 
-	if(g_ascii_strncasecmp("file:", uri, 5) == 0) {
-                win = GGV_WINDOW(ggv_window_new());
-                ggv_window_load(win, uri + 5);
-                gtk_widget_show(GTK_WIDGET(win));
-        }
+        win = GGV_WINDOW(ggv_window_new());
+        ggv_window_load(win, uri);
+        gtk_widget_show(GTK_WIDGET(win));
 }
 
 static void
@@ -732,7 +682,7 @@ ggv_window_new(void)
         }
         CORBA_exception_free(&ev);
 
-	win = GGV_WINDOW(g_object_new(GGV_TYPE_WINDOW, "win_name", "ggv", "title", _("Gnome Ghostview"), NULL));
+	win = GGV_WINDOW(g_object_new(GGV_TYPE_WINDOW, "win_name", "ggv", "title", _("GGV: no document loaded"), NULL));
         win->control = control;
 
         win->current_page = -2;
@@ -870,16 +820,12 @@ ggv_window_load(GgvWindow *win, const gchar *filename)
 {
         CORBA_Environment ev;
         Bonobo_PersistFile pf;
-        gchar *full_path;
+        gchar *title;
 
         g_return_val_if_fail(win != NULL, FALSE);
         g_return_val_if_fail(GGV_IS_WINDOW(win), FALSE);
         g_return_val_if_fail(filename != NULL, FALSE);
 
-        /* TODO: add handling of stdin: should probably be solved by 
-           copying stdin to a stream. */
-        g_assert(strcmp(filename, "-"));
-
         CORBA_exception_init(&ev);
         pf = Bonobo_Unknown_queryInterface(win->control,
                                            "IDL:Bonobo/PersistFile:1.0", &ev);
@@ -888,27 +834,16 @@ ggv_window_load(GgvWindow *win, const gchar *filename)
                 return FALSE;
         }
 
-        /* we need to pass full path to file to the control as we
-           do not really know where it resides.
-           NOTE: this still only works when the control is running
-           on the same computer as the GGV. I suppose this will mostly
-           be the case ;) */
-        if(*filename != '/') {
-                gchar *cwd;
-
-                cwd = g_get_current_dir();
-                full_path = g_strconcat(cwd, "/", filename, NULL);
-                g_free(cwd);
-        }
-        else
-                full_path = g_strdup(filename);
-
-        Bonobo_PersistFile_load(pf, full_path, &ev);
+        Bonobo_PersistFile_load(pf, filename, &ev);
         bonobo_object_release_unref(pf, NULL);
 
         if(win->filename)
                 g_free(win->filename);
-        win->filename = full_path;
+        win->filename = g_strdup(filename);
+
+        title = g_strconcat(_("GGV: "), win->filename, NULL);
+        gtk_window_set_title(GTK_WINDOW(win), title);
+        g_free(title);
 
         if(BONOBO_EX(&ev)) {
                 CORBA_exception_free(&ev);
