@@ -13,10 +13,11 @@ extern const gint ggv_max_orientation_labels;
 extern const gfloat ggv_unit_factors[];
 extern const gchar *ggv_unit_labels[];
 extern const gint ggv_max_unit_labels;
-extern const gfloat ggv_zoom_levels[];
+extern gfloat ggv_zoom_levels[];
 extern const gchar *ggv_zoom_level_names[];
 extern const gint ggv_max_zoom_levels;
 
+/* zoom level index <-> zoom factor */
 gint ggv_zoom_index_from_float (gfloat zoom_level);
 gfloat ggv_zoom_level_from_index (gint index);
 
@@ -29,6 +30,9 @@ gint ggv_get_index_of_string(gchar *string, gchar **strings);
 /* Quote filename for system call */
 gchar *ggv_quote_filename (const gchar *str);
 
+/* escape filename to conform to URI specification */
+gchar *ggv_escape_filename(const gchar *fname);
+
 /* If file exists and is a regular file then return its length, else -1 */
 gint ggv_file_length (const gchar *filename);
 
@@ -38,6 +42,7 @@ gboolean ggv_file_readable (const char *filename);
 /* Set a tooltip for a widget */
 void ggv_set_tooltip(GtkWidget* w, const gchar* tip);
 
+/* zoom <-> magstep (currently not used...) */
 gfloat ggv_compute_zoom(gint zoom_spec);
 gint ggv_compute_spec(gfloat zoom);
 
